import { isArray, isFn } from './type.js'
import { last } from './list.js'
import { curryN } from './curry.js'
/**
 * @param {PropertyKey[]} methods 
 * @param {import('./types.js').Fn} f
 */
export function dispatch(methods, f) {
  return curryN(f.length, function() {
    if (arguments.length === 0) {
      return f()
    }

    const args = Array.from(arguments)
    const obj = last(args)

    if (!isArray(obj)) {
      const len = methods.length
      for (let i = 0; i < len; i++) {
        const fn = obj[methods[i]]
        if (isFn(fn)) {
          return fn.apply(obj, args.slice(0, -1))
        }
      }
    }

    return f.apply(this, arguments)
  })
}

